@@ -1,9 +1,16 @@
-sim2.bd.rateshift.single <- function(n,lambdavec,muvec,frac,times){
+sim2.bd.rateshift.single <- function(n,lambdavec,muvec,frac,times,K,norm=TRUE){
 	check <- 0
 	sumratiolam <- 0
 	for (j in 1:length(lambdavec)){
-		sumratiolam <- sumratiolam + 1/lambdavec[j]
-		}
+		if (lambdavec[j]>0){
+		sumratiolam <- sumratiolam + 1/lambdavec[j]}
+	}
+	
+	maxleaf <- round(n/frac[1])
+	if (K>0 && (K<=maxleaf || muvec[1]>0)){
+		print("The carrying capacity K may not be smaller than the number of extant species round(n/frac[1])! In the current implementation, mu=c(0) is required.")
+		phy3<-list()
+	} else {
 	while (check ==0){
 	maxleaf <- round(n/frac[1])
 	leaves<- 1:maxleaf
@@ -17,21 +24,27 @@ sim2.bd.rateshift.single <- function(n,lambdavec,muvec,frac,times){
 	extinct<-0
 	doneME <-0
 	lambda <- lambdavec[1]
+	lambda0<-lambda
+	if (K>0){lambda<-max(lambda0*(1-length(leaves)/K),0)}
 	mu<-muvec[1]
 	if (length(frac)==1){
 		doneME<-1
 		nextextinction <- 0
 		} else {
-		nextextinction=times[interval]	
+		nextextinction<-times[interval]	
 			}
 	while (length(leaves)>0){
-	timestep <-rexp(1,(length(leaves)*(lambda+mu)))
-	time <- time+timestep
+	if ((lambda+mu)==0){
+		time <- nextextinction
+		} else {		
+		timestep <- rexp(1,(length(leaves)*(lambda+mu)))
+		time <- time+timestep
+		}
 	
 	if (nextextinction>time || doneME==1){
 	timecreation <- c(timecreation,time)
 	specevent <- runif(1,0,1)   #event speciation or extinction
-	if ((lambda/(lambda+mu)) >= specevent) { #speciation
+	if ((lambda/(lambda+mu)) >= specevent || K>0) { #speciation
 		if (length(leaves)>1) {
 		newspecies<- newspecies-1
 		nodes<-c(nodes,newspecies)
@@ -59,10 +72,12 @@ sim2.bd.rateshift.single <- function(n,lambdavec,muvec,frac,times){
 		edge.length<- c(edge.length,(time-timecreation[spec]))
 		leaves <- leaves[-1]
 		}
+		if (K>0){lambda<-max(lambda0*(1-length(leaves)/K),0)}
 		} else {
 			extinct <- extinct+1
 			leaves<- c(leaves,(maxleaf+extinct))
 			nodes<-c(nodes,(maxleaf+extinct))
+			if (K>0){lambda<-max(lambda0*(1-length(leaves)/K),0)}
 			}
 		} else {
 		#mass extinction event
@@ -77,6 +92,8 @@ sim2.bd.rateshift.single <- function(n,lambdavec,muvec,frac,times){
 			}
 			}
 		lambda<-lambdavec[interval]
+		lambda0<-lambda
+		if (K>0){lambda<-max(lambda0*(1-length(leaves)/K),0)}
 		mu<-muvec[interval]
 		if (interval == length(frac)){
 			doneME<-1
@@ -86,13 +103,16 @@ sim2.bd.rateshift.single <- function(n,lambdavec,muvec,frac,times){
 				}
 		}
 	}
-	sampletree <- runif(1,0,1)
+	
+	if (norm==TRUE) {sampletree <- runif(1,0,1)
 	if (sampletree < (1/lambda/(sumratiolam))){
 		check <- 1
-		}
+		} } else {check<-1}
 	}
 
 
+if (length(edge.length)==1){phy3<-list(1,time)} else {
+
 minval <- -min(edge)
 for (j in 1:length(edge.length)){
 	if (edge[j,1]<0){
@@ -112,6 +132,6 @@ class(phy) <- "phylo"
 #phy2<-phy
 phy2<-collapse.singles(phy)
 phy2<-reorder(phy2)
-phy3<-list(phy2,time)
+phy3<-list(phy2,time)}}
 phy3
 }	
